@@ -1,3 +1,5 @@
+from squyrrel.db.exceptions import DatabaseConnectionException
+
 
 class SqlDatabaseConnection:
 
@@ -9,6 +11,8 @@ class SqlDatabaseConnection:
         self._cursor = None
 
     def create_cursor(self, *args, **kwargs):
+        if self.c is None:
+            raise DatabaseConnectionException('Database not connected!')
         self._cursor = self.c.cursor(*args, **kwargs)
         return self._cursor
 
@@ -21,8 +25,9 @@ class SqlDatabaseConnection:
         raise NotImplementedError
 
     def close_connection(self):
-        if self.c:
-            self.c.close()
+        if self.c is None:
+            raise DatabaseConnectionException('Database not connected!')
+        self.c.close()
 
     def close(self):
         self.close_cursor()
@@ -39,9 +44,13 @@ class SqlDatabaseConnection:
             self._cursor.execute(sql)
 
     def commit(self):
+        if self.c is None:
+            raise DatabaseConnectionException('Database not connected!')
         self.c.commit()
 
     def rollback(self):
+        if self.c is None:
+            raise DatabaseConnectionException('Database not connected!')
         self.c.rollback()
 
     def fetchall(self):
