@@ -8,7 +8,8 @@ from squyrrel.orm.field import (ManyToOne, ManyToMany, OneToMany)
 from squyrrel.orm.filter import (ManyToOneFilter, ManyToManyFilter)
 from squyrrel.orm.model import Model
 from squyrrel.orm.signals import model_loaded_signal
-from squyrrel.orm.utils import sanitize_id_array, m2m_aggregation_subquery_alias, field_to_sql_data_type
+from squyrrel.orm.utils import sanitize_id_array, m2m_aggregation_subquery_alias, field_to_sql_data_type, \
+    extract_entities_without_id, extract_entity_with_id
 from squyrrel.sql.query import (Query, UpdateQuery, InsertQuery,
                                 DeleteQuery, CreateTableQuery)
 from squyrrel.sql.clauses import *
@@ -766,28 +767,43 @@ class QueryWizzard:
                     foreign_id_field_name: id_to_add
                 })
             )
-        for ids_to_remove in ids_to_remove:
+        for id_to_remove in ids_to_remove:
             condition = And.concat([
                 Equals(ColumnReference(id_field_name, table), Parameter(instance_id)),
-                Equals(ColumnReference(foreign_id_field_name, table), Parameter(ids_to_remove))
+                Equals(ColumnReference(foreign_id_field_name, table), Parameter(id_to_remove))
             ])
             queries.append(DeleteQuery(table, condition))
 
         # todo: delete in one query with 'IN' expression
         return queries
 
-    def build_one_to_many_update_queries(self, model, instance_id, relation, related_ids):
+    def build_one_to_many_update_queries(self, model, instance_id, relation: OneToMany, current_entities, new_related_entities):
         model = self.get_model(model)
         foreign_model = self.get_model(relation.foreign_model)
+        foreign_id_field_name = foreign_model.id_field_name()
         queries = []
-        updates = {model.id_field_name(): instance_id}
-        for related_id in related_ids:
-            filter_condition = Equals.id_as_parameter(foreign_model, related_id)
-            queries.append(
-                UpdateQuery.build(foreign_model,
-                                  filter_condition=filter_condition,
-                                  updates=updates)
-            )
+        new_entities = extract_entities_without_id(foreign_model, new_related_entities)
+        current_ids = set(sanitize_id_array(foreign_model, current_entities))
+        update_ids = set(sanitize_id_array(foreign_model, new_related_entities))
+        ids_to_remove = current_ids - update_ids
+        table = foreign_model.table_name
+
+        for id_to_remove in ids_to_remove:
+            condition = Equals(ColumnReference(foreign_id_field_name, table),
+                               Parameter(id_to_remove))
+            queries.append(DeleteQuery(table, condition))
+
+        for update_id in update_ids:
+            filter_condition = Equals.id_as_parameter(foreign_model, update_id)
+            prev_data = extract_entity_with_id(foreign_model, current_entities, update_id)
+            data = extract_entity_with_id(foreign_model, new_related_entities, update_id)
+            queries.extend(self.update_queries(foreign_model, filter_condition, update_id, prev_data, data, False))
+
+        for new_entity in new_entities:
+            id_field_name = model.id_field_name()
+            new_entity[id_field_name] = instance_id
+            queries.append(self.insert_query(foreign_model, new_entity))
+
         return queries
 
     def delete_by_condition(self, model, filter_condition, commit=True):
@@ -825,7 +841,8 @@ class QueryWizzard:
                             model=model,
                             instance_id=instance_id,
                             relation=relation,
-                            related_ids=value
+                            current_entities=[],
+                            new_related_entities=value
                         )
                     )
         return related_queries
@@ -851,12 +868,13 @@ class QueryWizzard:
         inserts = dict()
         for column, value in data.items():
             field = model.get_field(column)
+            # TODO: compare with logic in update_queries
             if field is None:
                 if not value:
                     continue
                 relation = model.get_relation(column)
                 if isinstance(relation, ManyToOne):
-                    fk_id = self.get_m21_value(relation, relation.insert_search_column, value)
+                    fk_id = self.get_fk_id(relation, value, relation.insert_search_column)
                     if fk_id:
                         inserts[relation.foreign_key_field] = fk_id
             else:
@@ -875,6 +893,14 @@ class QueryWizzard:
         return InsertQuery.build(
             table=model.table_name, inserts=inserts)
 
+    def get_fk_id(self, relation, value, search_column):
+        if isinstance(value, dict):
+            relation_model = self.get_model(relation.foreign_model)
+            fk_id = value[relation_model.id_field_name()]
+        else:
+            fk_id = self.get_m21_value(relation, search_column, value)
+        return fk_id
+
     def merge(self, model, new_instance_data, old_instance_ids, return_created_object=True):
         model = self.get_model(model)
 
@@ -947,14 +973,17 @@ class QueryWizzard:
             return self.get_by_id(model, inserted_id, entity_format=EntityFormat.JSON)
         return inserted_id
 
-    def update(self, model, filter_condition, instance_id, prev_data, data):
+    def update_queries(self, model, filter_condition, instance_id, prev_data, data, include_to_many_queries=True):
         # todo only update changed data
         # i.e. difference data - instance.data
         # todo: what about commit?
         model = self.get_model(model)
         updates = dict()
         m2m_update_queries = []
-
+        one_to_many_queries = []
+        if not include_to_many_queries:
+            print('prev_data', prev_data)
+            print('data', data)
         for column, value in data.items():
             # todo value into sql value
             field = model.get_field(column)
@@ -967,11 +996,13 @@ class QueryWizzard:
             relation = model.get_relation(column)
 
             if isinstance(relation, ManyToOne):
-                fk_id = self.get_m21_value(relation, relation.update_search_column, value)
-                if fk_id:
-                    updates[relation.foreign_key_field] = fk_id
-
-            elif isinstance(relation, ManyToMany):
+                if value is None:
+                    updates[relation.foreign_key_field] = None
+                else:
+                    fk_id = self.get_fk_id(relation, value, relation.update_search_column)
+                    if fk_id:
+                        updates[relation.foreign_key_field] = fk_id
+            elif isinstance(relation, ManyToMany) and include_to_many_queries:
                 # compare difference
                 # if instance is None, needs to be loaded first..
                 m2m_update_queries.extend(
@@ -983,19 +1014,33 @@ class QueryWizzard:
                         new_ids=value
                     )
                 )
+            elif isinstance(relation, OneToMany) and include_to_many_queries:
+                print('current_entities:', prev_data.get(column))
+                # in value the id is missing!
+                print('new_related_entities:', value)
+                one_to_many_queries.extend(
+                    self.build_one_to_many_update_queries(
+                        model=model,
+                        instance_id=instance_id,
+                        relation=relation,
+                        current_entities=prev_data.get(column),
+                        new_related_entities=value
+                    )
+                )
             elif relation is None:
                 raise Exception(f'Error during update: Did not find column {column}')
             else:
                 raise Exception(f'Error during update: Could not handle relation {repr(relation)}')
-
         update_query = UpdateQuery.build(
             model, filter_condition=filter_condition, updates=updates)
+        return [update_query] + m2m_update_queries + one_to_many_queries
 
-        self.execute_queries_in_transaction(queries=[update_query] + m2m_update_queries)
+    def update(self, model, filter_condition, instance_id, prev_data, data):
+        queries = self.update_queries(model, filter_condition, instance_id, prev_data, data)
+        self.execute_queries_in_transaction(queries)
 
     def update_by_id(self, model, instance_id, data, prev_data, commit=True, return_updated_object=True):
         # todo: add extensive doc string for params!!
-
         model = self.get_model(model)
         filter_condition = Equals.id_as_parameter(model, instance_id)
         # todo: add logging
