@@ -1,7 +1,9 @@
 """
 PostgreSQL: https://www.postgresql.org/docs/9.5/sql-select.html
 """
-
+from squyrrel.orm.field import ForeignKey
+from squyrrel.orm.model import Model
+from squyrrel.orm.utils import field_to_sql_data_type
 from squyrrel.sql.clauses import (UpdateClause, WhereClause, SetClause,
                                   InsertClause, ValuesClause, DeleteClause, SelectClause)
 from squyrrel.sql.expressions import Parameter
@@ -67,6 +69,8 @@ class Query:
             output = f'({output})'
             if self.alias is not None:
                 output += f' AS {self.alias}'
+        if not self.is_subquery:
+            output += ';'
         return output
 
 
@@ -76,11 +80,19 @@ class ColumnDefinition:
                  name,
                  data_type,
                  primary_key=False,
-                 not_null=False):
+                 foreign_key: ForeignKey = None,
+                 not_null=False,
+                 default=None,
+                 unique=False,
+                 collate=None):
         self.name = name
         self.data_type = data_type
         self.primary_key = primary_key
+        self.foreign_key = foreign_key
         self.not_null = not_null
+        self.default = default
+        self.unique = unique
+        self.collate = collate
 
     def __repr__(self):
         output = f'{self.name} {self.data_type}'
@@ -88,6 +100,14 @@ class ColumnDefinition:
             output += ' PRIMARY KEY'
         if self.not_null:
             output += ' NOT NULL'
+        if self.default:
+            output += f' DEFAULT {repr(self.default)}'
+        if self.unique:
+            output += ' UNIQUE'
+        if self.collate is not None:
+            output += f' COLLATE {self.collate}'
+        if self.foreign_key is not None:
+            output += f' {repr(self.foreign_key)}'
         return output
 
 
@@ -108,12 +128,16 @@ class CreateTableQuery:
         col1 data_type PRIMARY KEY,
         col2 data_type NOT NULL,
         ...
+        UNIQUE (col1, col2)
+        PRIMARY KEY (col5, col6)
     );
     """
 
-    def __init__(self, create_clause, column_definitions):
+    def __init__(self, create_clause, column_definitions, uniqueness_constraints=None, pk_constraints=None):
         self.create_clause = create_clause
         self.column_definitions = column_definitions
+        self.uniqueness_constraints = uniqueness_constraints
+        self.pk_constraints = pk_constraints
 
     def __repr__(self):
         clause_separation = '\n'
@@ -123,6 +147,25 @@ class CreateTableQuery:
         for col in self.column_definitions:
             col_outputs.append(repr(col))
         output += (',' + clause_separation).join(col_outputs)
+
+        # table constraints:
+        constraint_outputs = []
+        if self.uniqueness_constraints is not None:
+            # todo: refactor: create class UniquenessConstraint (for table and for column)
+            for uniqueness_constraint in self.uniqueness_constraints:
+                constraint_columns = ', '.join(uniqueness_constraint)
+                constraint = f'UNIQUE ({constraint_columns})'
+                constraint_outputs.append(constraint)
+        if self.pk_constraints is not None:
+            # todo: refactor: create class UniquenessConstraint (for table and for column)
+            for pk_constraint in self.pk_constraints:
+                constraint_columns = ', '.join(pk_constraint)
+                constraint = f'PRIMARY KEY ({constraint_columns})'
+                constraint_outputs.append(constraint)
+        if constraint_outputs:
+            output += (',' + clause_separation)
+            output += (',' + clause_separation).join(constraint_outputs)
+
         output += ');'
         return output
 
@@ -131,7 +174,7 @@ class CreateTableQuery:
         return None
 
     @classmethod
-    def build(cls, table, columns, if_not_exists=False):
+    def build(cls, table, columns, if_not_exists=False, uniqueness_constraints=None):
         create_clause = CreateTableClause(table, if_not_exists=if_not_exists)
         column_definitions = []
         for column_name, column_kwargs in columns.items():
@@ -139,9 +182,42 @@ class CreateTableQuery:
                 name=column_name,
                 data_type=column_kwargs['data_type'],
                 primary_key=column_kwargs.get('primary_key', False),
-                not_null=column_kwargs.get('not_null', False)
+                foreign_key=column_kwargs.get('foreign_key', None),
+                not_null=column_kwargs.get('not_null', False),
+                default=column_kwargs.get('default', None),
+                unique=column_kwargs.get('unique', False),
+                collate=column_kwargs.get('collate', None)
             ))
-        return cls(create_clause, column_definitions=column_definitions)
+        return cls(create_clause, column_definitions=column_definitions, uniqueness_constraints=uniqueness_constraints)
+
+
+class CreateManyToManyJunctionTableQuery(CreateTableQuery):
+
+    def __init__(self, model1: Model, model2: Model, junction_table_name, if_not_exists=False):
+        field_name1 = model1.id_field_name()
+        field_name2 = model2.id_field_name()
+        fk1 = ForeignKey(model=model1,
+                         foreign_field=field_name1,
+                         on_delete=ForeignKey.ReferentialAction.CASCADE)
+        fk2 = ForeignKey(model=model2,
+                         foreign_field=field_name2,
+                         on_delete=ForeignKey.ReferentialAction.CASCADE)
+        column_definitions = [
+            ColumnDefinition(name=field_name1,
+                             data_type=field_to_sql_data_type(model1.id_field()),
+                             primary_key=False,
+                             foreign_key=fk1,
+                             not_null=True),
+            ColumnDefinition(name=field_name2,
+                             data_type=field_to_sql_data_type(model2.id_field()),
+                             primary_key=False,
+                             foreign_key=fk2,
+                             not_null=True),
+        ]
+        primary_key_constraint = (field_name1, field_name2)
+        super().__init__(create_clause=CreateTableClause(junction_table_name, if_not_exists=if_not_exists),
+                         column_definitions=column_definitions,
+                         pk_constraints=[primary_key_constraint])
 
 
 class InsertQuery:
@@ -163,6 +239,7 @@ class InsertQuery:
         output = repr(self.insert_clause)
         output += clause_separation
         output += repr(self.values_clause)
+        output += ';'
         return output
 
     @classmethod
@@ -174,6 +251,31 @@ class InsertQuery:
         return instance
 
 
+class BatchInsertQuery(InsertQuery):
+    """
+    INSERT INTO table_name (col1, col2, ...)
+    VALUES
+    (val1, val2, ...)
+    (val1, val2, ...)
+    """
+
+    def __init__(self, table, insert_clause=None, rows=None):
+        super().__init__(table, insert_clause=insert_clause)
+        self.rows = rows
+
+    def __repr__(self):
+        clause_separation = '\n'
+        output = repr(self.insert_clause)
+        output += clause_separation
+        output += 'VALUES' + clause_separation
+
+        for row in self.rows[:-1]:
+            output += '({}),\n'.format(', '.join(repr(value) for value in row))
+        output += '({})\n'.format(', '.join(repr(value) for value in self.rows[-1]))
+        output += ';'
+        return output
+
+
 class DeleteQuery:
     """
     DELETE FROM table_name
@@ -195,6 +297,7 @@ class DeleteQuery:
         output = repr(self.delete_clause)
         output += clause_separation
         output += repr(self.where_clause)
+        output += ';'
         return output
 
 
@@ -205,7 +308,7 @@ class UpdateQuery:
     WHERE condition;
     """
 
-    def __init__(self, update_clause, set_clause, where_clause):
+    def __init__(self, update_clause: UpdateClause, set_clause: SetClause, where_clause: WhereClause):
         self.update_clause = update_clause
         self.set_clause = set_clause
         self.where_clause = where_clause
@@ -217,6 +320,7 @@ class UpdateQuery:
         output += repr(self.set_clause)
         output += clause_separation
         output += repr(self.where_clause)
+        output += ';'
         return output
 
     @property
@@ -235,8 +339,3 @@ class UpdateQuery:
             set_clause=SetClause(**updates),
             where_clause=WhereClause(filter_condition)
         )
-
-# class QueryBuilder:
-
-#     def __init__(self):
-#         self.query = None
