@@ -5,7 +5,7 @@ from squyrrel.orm.exceptions import RelationNotFoundException
 from squyrrel.sql.join import OnJoinCondition, JoinType
 from squyrrel.sql.utils import sanitize_column_reference, listify
 from squyrrel.orm.filter import ManyToOneFilter, ManyToManyFilter, StringFieldFilter, OrFieldFilter, FieldFilter, \
-    BooleanFieldFilter, JunctionType
+    BooleanFieldFilter, JunctionType, OneToManyFilter
 from squyrrel.sql.references import ColumnReference
 from squyrrel.orm.model import Model
 from squyrrel.sql.clauses import SelectClause, FromClause, WhereClause, Pagination, OrderByClause
@@ -105,6 +105,15 @@ class QueryBuilder:
             filter.value
         )
 
+    def one_to_many_filter_condition(self, filter: OneToManyFilter):
+        filter_foreign_model = self._get_model(filter.foreign_model)
+        # We assume that filter.value contains only one id value
+        # TODO: extend to array as in many_to_many_filter_conditions()
+        return Equals.column_as_parameter(
+            ColumnReference(filter.key, table=filter_foreign_model.table_name),
+            filter.value
+        )
+
     def many_to_many_filter_condition(self, filter: ManyToManyFilter):
         # todo: handle filter.negate
         conditions = []
@@ -153,6 +162,8 @@ class QueryBuilder:
     def filter_condition(self, filter: Type[FieldFilter]):
         if isinstance(filter, ManyToOneFilter):
             return self.many_to_one_filter_condition(filter)
+        elif isinstance(filter, OneToManyFilter):
+            return self.one_to_many_filter_condition(filter)
         elif isinstance(filter, ManyToManyFilter):
             return self.many_to_many_filter_condition(filter)
         elif isinstance(filter, StringFieldFilter):
@@ -248,7 +259,6 @@ class QueryBuilder:
     def _build_where_clause(self):
         if self._filter_conditions:
             self._where_clause = WhereClause(And.concat(self._filter_conditions))
-            # print(self._where_clause)
         else:
             self._where_clause = None
 
@@ -309,6 +319,9 @@ class QueryBuilder:
             # print('include_many_to_many_join:', relation_name)
             self._include_many_to_many_join(relation=relation)
 
+        for relation_name, relation in self._one_to_many_relations:
+            self._include_one_to_many_join(relation=relation)
+
     def _include_column(self, column_reference):
         # print('model, table_name', self.model.table_name)
         # print('include_column', column_reference.table, self._model.table_name)
@@ -328,11 +341,22 @@ class QueryBuilder:
                     self._add_one_to_many_relation(foreign_model=foreign_model,
                                                    relation_name=relation_name)
 
-    # todo: def _include_many_to_one_join()
+    def _include_one_to_many_join(self, relation):
+        # todo: refactor: compare with QueryWizzard.include_many_to_many_join
+        # !! todo: first check if not already joined!!
+        foreign_model = self._get_model(relation.foreign_model)
+        foreign_table = foreign_model.table_name
+        join_condition = OnJoinCondition(
+            Equals(ColumnReference(self._model.id_field_name(), table=self._model.table_name),
+                   ColumnReference(self._model.id_field_name(), table=foreign_table))
+        )
+        self._join_to_from_clause(
+            join_type=JoinType.INNER_JOIN,
+            table=foreign_table,
+            join_condition=join_condition)
 
     def _include_many_to_many_join(self, relation):
         # todo: refactor: compare with QueryWizzard.include_many_to_many_join
-
         # !! todo: first check if not already joined!!
 
         # foreign_model = self.qw.get_model(relation.foreign_model)
